@@ -1,7 +1,7 @@
 /* GENIUS Calculator
- * Copyright (C) 1997-2002 George Lebl
+ * Copyright (C) 1997-2004 Jiri (George) Lebl
  *
- * Author: George Lebl
+ * Author: Jiri (George) Lebl
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,8 +19,8 @@
  * USA.
  */
 
-#ifndef _MPWRAP_H_
-#define _MPWRAP_H_
+#ifndef MPWRAP_H_
+#define MPWRAP_H_
 
 #ifdef HAVE_GMP2_INCLUDE_DIR
 #include <gmp2/gmp.h>
@@ -28,32 +28,48 @@
 #include <gmp.h>
 #endif
 
-#define MPW_NATIVEINT	1
-#define MPW_INTEGER	2
-#define MPW_RATIONAL	3
-#define MPW_FLOAT	4
+/* FIXME: we may need the same as above */
+#ifdef HAVE_MPFR
+#include <mpfr.h>
+#include <mpf2mpfr.h>
 
-#define MPW_REAL	1
-#define MPW_COMPLEX	2
+/* FIXME: ugly hack to get pi_mpf and the structs to work */
+#define mpf_ptr mpfr_ptr
+#define __mpf_struct __mpfr_struct
+#endif
+
+
+enum {
+	MPW_INTEGER = 1,
+	MPW_RATIONAL,
+	MPW_FLOAT
+};
+
+enum {
+	MPW_REAL = 1,
+	MPW_COMPLEX
+};
+
+/* FIXME: we really have too many structures/pointers per number here
+   that should be remedied */
 
 /*number structures, this is where low level stuff is stored so it will be
   different for each lib, members should never be directly accessed!*/
 
 /*real only structure*/
 typedef struct _MpwRealNum {
-	int type;
 	struct { /*this is done as a struct so that conversions don't require
 		   a temporary, make sure to clear the old one!, in the worst
 		   case we have two unused pointers*/
 		mpz_ptr ival;
 		mpq_ptr rval;
 		mpf_ptr fval;
-		long nval;
 	} data;
 	union {
 		struct _MpwRealNum *next; /*used for free lists*/
 		int usage; /*used for copy-on-write*/
 	} alloc; /*private union for memory managment stuff*/
+	guint8 type;
 } MpwRealNum;
 
 /*any number (includes complex) so it includes an imaginary member if type
@@ -61,28 +77,33 @@ typedef struct _MpwRealNum {
 
   this is used as the number type*/
 struct _mpw_t {
-	int type;
 	MpwRealNum *r; /*real*/
 	MpwRealNum *i; /*imaginary*/
+	guint8 type;
 };
 
 typedef struct _mpw_t mpw_t[1];
 typedef struct _mpw_t *mpw_ptr;
 
+/* FIXME: this is evil, error_num is used elsewhere, should
+ * be some more generalized error interface */
 enum {
 	MPW_NO_ERROR=0,
 	MPW_INTERNAL_ERROR,
 	MPW_NUMERICAL_ERROR,
 };
 
-/* private struct, use accessors for fields */
-typedef struct _MpwCtx MpwCtx;
 
 #if 0
+/* If we ever will need different contexts, currently very much on the
+   backburner */
 /*************************************************************************/
 /*conext level stuff                                                     */
 /*************************************************************************/
 
+/* private struct, use accessors for fields */
+typedef struct _MpwCtx MpwCtx;
+
 typedef void (*MpwErrorFunc) (MpwCtx *, char *);
 
 /* make new context with a refcount of 1 */
@@ -127,8 +148,31 @@ void mpw_make_same_type(mpw_ptr op1,mpw_ptr op2);
 
 void mpw_set(mpw_ptr rop,mpw_ptr op);
 void mpw_set_d(mpw_ptr rop,double d);
+void mpw_set_d_complex (mpw_ptr rop, double real, double imag);
 void mpw_set_si(mpw_ptr rop,signed long int i);
 void mpw_set_ui(mpw_ptr rop,unsigned long int i);
+void mpw_set_mpz_use (mpw_ptr rop, mpz_ptr op);
+void mpw_set_mpq_use (mpw_ptr rop, mpq_ptr op);
+void mpw_set_mpf_use (mpw_ptr rop, mpf_ptr op);
+
+mpz_ptr mpw_peek_real_mpz (mpw_ptr op);
+mpq_ptr mpw_peek_real_mpq (mpw_ptr op);
+mpf_ptr mpw_peek_real_mpf (mpw_ptr op);
+
+mpz_ptr mpw_peek_imag_mpz (mpw_ptr op);
+mpq_ptr mpw_peek_imag_mpq (mpw_ptr op);
+mpf_ptr mpw_peek_imag_mpf (mpw_ptr op);
+
+/* Just quick hacks to get an mpz, tmp should be an unused mpz_t,
+   rop should be mpz_ptr and op should be mpw_ptr */
+#define MPW_MPZ_REAL(rop,op,tmp) { \
+				   rop = mpw_peek_real_mpz (op); \
+			       }
+#define MPW_MPZ_IMAG(rop,op,tmp) {  \
+				   rop = mpw_peek_imag_mpz (op); \
+			       }
+#define MPW_MPZ_KILL(rop,tmp) { if (rop == tmp) mpz_clear (tmp); }
+
 
 void mpw_abs(mpw_ptr rop,mpw_ptr op);
 
@@ -160,16 +204,21 @@ void mpw_legendre(mpw_ptr rop,mpw_ptr op1, mpw_ptr op2);
 void mpw_kronecker(mpw_ptr rop,mpw_ptr op1, mpw_ptr op2);
 void mpw_lucnum (mpw_ptr rop, mpw_ptr op);
 void mpw_nextprime (mpw_ptr rop, mpw_ptr op);
-int mpw_probab_prime_p (mpw_ptr op, mpw_ptr reps);
 gboolean mpw_perfect_square(mpw_ptr op);
 gboolean mpw_perfect_power(mpw_ptr op);
+gboolean mpw_even_p(mpw_ptr op);
+gboolean mpw_odd_p(mpw_ptr op);
 
 void mpw_pow(mpw_ptr rop,mpw_ptr op1, mpw_ptr op2);
 void mpw_pow_ui(mpw_ptr rop,mpw_ptr op, unsigned long int e);
+void mpw_powm(mpw_ptr rop,mpw_ptr op1, mpw_ptr op2, mpw_ptr mod);
+void mpw_powm_ui(mpw_ptr rop,mpw_ptr op, unsigned long int e, mpw_ptr mod);
 void mpw_sqrt(mpw_ptr rop,mpw_ptr op);
 
 void mpw_exp(mpw_ptr rop,mpw_ptr op);
 void mpw_ln(mpw_ptr rop,mpw_ptr op);
+void mpw_log2(mpw_ptr rop,mpw_ptr op);
+void mpw_log10(mpw_ptr rop,mpw_ptr op);
 
 void mpw_sin(mpw_ptr rop,mpw_ptr op);
 void mpw_cos(mpw_ptr rop,mpw_ptr op);
@@ -177,6 +226,7 @@ void mpw_sinh(mpw_ptr rop,mpw_ptr op);
 void mpw_cosh(mpw_ptr rop,mpw_ptr op);
 void mpw_arctan(mpw_ptr rop,mpw_ptr op);
 void mpw_pi (mpw_ptr rop);
+void mpw_euler_constant (mpw_ptr rop);
 void mpw_i (mpw_ptr rop);
 void mpw_rand (mpw_ptr rop);
 void mpw_randint (mpw_ptr rop, mpw_ptr op);
@@ -186,8 +236,8 @@ void mpw_conj (mpw_ptr rop, mpw_ptr op);
 int mpw_cmp(mpw_ptr op1, mpw_ptr op2);
 int mpw_cmp_ui(mpw_ptr op, unsigned long int i);
 
-int mpw_eql(mpw_ptr op1, mpw_ptr op2);
-int mpw_eql_ui(mpw_ptr op, unsigned long int i);
+gboolean mpw_eql(mpw_ptr op1, mpw_ptr op2);
+gboolean mpw_eql_ui(mpw_ptr op, unsigned long int i);
 
 void mpw_fac_ui(mpw_ptr rop,unsigned long int i);
 void mpw_fac(mpw_ptr rop,mpw_ptr op);
@@ -204,22 +254,34 @@ void mpw_make_float(mpw_ptr rop);
 void mpw_init_mp(void);
 
 /*get a string (g_malloc'ed) with the number in it*/
-char * mpw_getstring(mpw_ptr num,int max_digits,int scientific_notation,
-	int results_as_floats,int mixed_fractions, int integer_output_base);
+char * mpw_getstring (mpw_ptr num,
+		      int max_digits,
+		      gboolean scientific_notation,
+		      gboolean results_as_floats,
+		      gboolean mixed_fractions,
+		      /* FIXME: solve dependency, this is defined in calc.h */
+		      /* GelOutputStyle */int style,
+		      int integer_output_base,
+		      gboolean add_parenths);
 
 void mpw_set_str_float(mpw_ptr rop,const char *s,int base);
 
 /*reads only the imaginary part (use add for real part)*/
 void mpw_set_str_complex(mpw_ptr rop,const char *s,int base);
+void mpw_set_str_complex_int(mpw_ptr rop,const char *s,int base);
 
 void mpw_set_str_int(mpw_ptr rop,const char *s,int base);
 
 void mpw_set_str (mpw_ptr rop, const char *s, int base);
 
-int mpw_is_complex(mpw_ptr op);
-int mpw_is_integer(mpw_ptr op);
-int mpw_is_rational(mpw_ptr op);
-int mpw_is_float(mpw_ptr op);
+gboolean mpw_is_complex(mpw_ptr op);
+gboolean mpw_is_integer(mpw_ptr op);
+gboolean mpw_is_complex_integer(mpw_ptr op);
+gboolean mpw_is_rational(mpw_ptr op);
+gboolean mpw_is_rational_or_integer(mpw_ptr op);
+gboolean mpw_is_complex_rational_or_integer(mpw_ptr op);
+gboolean mpw_is_float(mpw_ptr op);
+gboolean mpw_is_complex_float(mpw_ptr op);
 
 void mpw_im(mpw_ptr rop, mpw_ptr op);
 void mpw_re(mpw_ptr rop, mpw_ptr op);
@@ -231,6 +293,8 @@ void mpw_trunc(mpw_ptr rop, mpw_ptr op);
 
 /*try to get a long number representation of the number*/
 long mpw_get_long(mpw_ptr op);
+/*try to get a double representation of the number*/
+double mpw_get_double(mpw_ptr op);
 
 void mpw_denominator(mpw_ptr rop, mpw_ptr op);
 void mpw_numerator(mpw_ptr rop, mpw_ptr op);
