@@ -1,7 +1,7 @@
 /* GENIUS Calculator
- * Copyright (C) 1997-2002 George Lebl
+ * Copyright (C) 1997-2004 Jiri (George) Lebl
  *
- * Author: George Lebl
+ * Author: Jiri (George) Lebl
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@ typedef enum {
 	GEL_USER_FUNC, /*function that points to an GelETree for evaluation*/
 	GEL_VARIABLE_FUNC, /*function that points to an GelETree result */
 	GEL_REFERENCE_FUNC /*a function that points to some other GelEFunc*/
-} GelEFuncType;
+} GelEFuncType; /* should fit in 3 bits (it does) */
 
 typedef struct _GelEFunc GelEFunc;
 typedef union _GelETree GelETree;
@@ -47,6 +47,8 @@ typedef struct _GelETreeString GelETreeString;
 typedef struct _GelETreeFunction GelETreeFunction;
 typedef struct _GelETreeComparison GelETreeComparison;
 typedef struct _GelETreeUsertype GelETreeUsertype;
+typedef struct _GelETreeBool GelETreeBool;
+
 typedef struct _GelETreeMatrixRow GelETreeMatrixRow;
 /*typedef struct _GelETreeMatrixStart GelETreeMatrixStart;*/
 /*typedef struct _GelETreeExprlistStart GelETreeExprlistStart;*/
@@ -69,6 +71,8 @@ typedef struct _GelMatrixW GelMatrixW;
 typedef GelETree *(* ParameterSetFunc) (GelETree *val);
 typedef GelETree *(* ParameterGetFunc) (void);
 
+typedef	GelETree *(* GelBIFunction) (GelCtx *ctx, GelETree * *, gboolean *); /*the gboolean is exception*/
+
 /* tokens point to this structure globaly, there is
    one such structure for each token.  */
 struct _GelToken {
@@ -81,33 +85,37 @@ struct _GelToken {
 	gpointer data1;
 	gpointer data2;
 
-	guint32 protected:1;
-	guint32 parameter:1;
-	guint32 built_in_parameter:1;
+	guint8 protected_:1;
+	guint8 parameter:1;
+	guint8 built_in_parameter:1;
 };
 
 struct _GelEFunc {
 	GelToken *id;
-	GelEFuncType type;
 	int context; /*the context number this is used for .. if we pop this
 		       context, we will destroy the function*/
-	int nargs; /*number of arguments*/
 	GSList *named_args; /*names of arguments*/
 
 	GSList *extra_dict;
 
 	union {
 		GelETree *user;
-		GelETree *(*func)(GelCtx *ctx, GelETree * *, int *); /*the integer is exception*/
+		GelBIFunction func;
 		GelEFunc *ref;
 		GelEFunc *next; /*this is for keeping a free list*/
 	} data;
+
+	guint16 nargs; /*number of arguments*/
+
+	/* GelEFuncType type; */
+	guint8 type:3;
+
 	/* if true, we must take this off the subst list for a context pop,
 	 * before we free the function */
-	guint32 on_subst_list:1;
-	guint32 vararg:1;
-	guint32 propagate_mod:1;
-	guint32 no_mod_all_args:1;
+	guint8 on_subst_list:1;
+	guint8 vararg:1;
+	guint8 propagate_mod:1;
+	guint8 no_mod_all_args:1;
 };
 
 typedef enum {
@@ -121,7 +129,8 @@ typedef enum {
 	STRING_NODE,
 	FUNCTION_NODE, /*stores an anonymous function*/
 	COMPARISON_NODE,
-	USERTYPE_NODE, /*for user types*/
+	USERTYPE_NODE, /*for user types, FIXME: not finished*/
+	BOOL_NODE, /*boolean*/
 	
 	/*marker nodes*/
 	MATRIX_ROW_NODE=1000,
@@ -150,7 +159,13 @@ struct _GelETreeMatrix {
 	GelETreeType type;
 	GelETree *next;
 	GelMatrixW *matrix;
+	gboolean quoted;
+	
+	/* gboolean is faster, then a bitfield and we right now
+	   don't gain anything */
+	/*
 	guint quoted:1;
+	*/
 };
 
 /* FIXME: Not implemented */
@@ -158,15 +173,29 @@ struct _GelETreeSet {
 	GelETreeType type;
 	GelETree *next;
 	GelETree *items;
+	gboolean multiset;
+
+	/* gboolean is faster, then a bitfield and we right now
+	   don't gain anything */
+	/*
 	guint multiset:1;
+	*/
 };
 
 struct _GelETreePolynomial {
 	GelETreeType type;
 	GelETree *next;
-	int arraysize;
-	int largest; /* largest exponent */
-	int vars; /* number of variables */
+	guint16 largest; /* largest exponent */
+	guint8 vars; /* number of variables */
+
+	/* This needs redoing.  No need to store
+	   this in the ETree struct since we want to
+	   conserve memory.  Probably need an array
+	   type for this. */
+	/*
+	guint16 arraysize;
+	*/
+
 	mpw_ptr *indexes; /* indexes when written out in standard form
 			     from smallest to largest.  If more then one
 			     variable then this is '(largest+1)^vars'
@@ -176,8 +205,8 @@ struct _GelETreePolynomial {
 struct _GelETreeOperator {
 	GelETreeType type;
 	GelETree *next;
-	int oper;
-	int nargs;
+	gint8 oper;
+	guint16 nargs;
 	GelETree *args;
 };
 
@@ -202,7 +231,7 @@ struct _GelETreeFunction {
 struct _GelETreeComparison {
 	GelETreeType type;
 	GelETree *next;
-	int nargs;
+	gint16 nargs;
 	GelETree *args;
 	GSList *comp;
 };
@@ -210,14 +239,25 @@ struct _GelETreeComparison {
 struct _GelETreeUsertype {
 	GelETreeType type;
 	GelETree *next;
-	int ttype;
+	gint16 ttype;
 	gpointer data;
 };
 
+struct _GelETreeBool {
+	GelETreeType type;
+	GelETree *next;
+	gboolean bool_;
+
+	/* gboolean is faster, then a bitfield and we right now
+	   don't gain anything */
+	/* guint bool_:1; */
+};
+
+
 struct _GelETreeMatrixRow {
 	GelETreeType type;
 	GelETree *next;
-	int nargs;
+	gint16 nargs;
 	GelETree *args;
 };
 
@@ -251,6 +291,7 @@ union _GelETree {
 	GelETreeFunction func;
 	GelETreeComparison comp;
 	GelETreeUsertype ut;
+	GelETreeBool bool_;
 	GelETreeMatrixRow row;
 	/*GelETreeMatrixStart mats;*/
 	/*GelETreeExprlistStart exps;*/
@@ -261,21 +302,30 @@ union _GelETree {
 
 /* The flag for the stack */
 enum {
-	GE_EMPTY_STACK,
-	GE_RESULT, /*used for recursive evluation with the same ctx*/
-	GE_PRE,
-	GE_POST,
-	GE_FUNCCALL,
-	GE_AND,
-	GE_OR,
-	GE_LOOP_LOOP,
-	GE_LOOP_COND,
-	GE_FOR,
-	GE_FORIN,
-	GE_MODULOOP,
-	GE_SETMODULO
+	GE_EMPTY_STACK = 0,
+	GE_RESULT      = 1, /*used for recursive evluation with the same ctx*/
+	GE_PRE         = 2,
+	GE_POST        = 3,
+	GE_FUNCCALL    = 4,
+	GE_AND         = 5,
+	GE_OR          = 6,
+	GE_LOOP_LOOP   = 7,
+	GE_LOOP_COND   = 8,
+	GE_FOR         = 9,
+	GE_FORIN       = 10,
+	GE_MODULOOP    = 11,
+	GE_SETMODULO   = 12,
+
+	/* flag mask */
+	GE_MASK        = 0xff,
+
+	/* bool flags */
+	GE_WHACKARG    = 1<<16 /* only on GE_PRE, GE_POST, GE_MODULOOP */
 };
 
+#define GE_ADDWHACKARG(flag,whackarg) \
+	((whackarg) ? ((flag) | GE_WHACKARG) : (flag))
+
 /*should take up about a page
   we will use a single pointer for data and the next pointer for post/pre flag
   we can use next as a marker that we have gone too far, and current as a marker
@@ -296,8 +346,8 @@ typedef enum {
 struct _GelEvalLoop {
 	GelETree * condition;
 	GelETree * body;
-	guint32 is_while:1; /*if false, this is an until loop*/
-	guint32 body_first:1; /*if true body is the first argument*/
+	guint8 is_while:1; /*if false, this is an until loop*/
+	guint8 body_first:1; /*if true body is the first argument*/
 };
 /*data structure for 'for' loops*/
 struct _GelEvalFor {
@@ -305,7 +355,7 @@ struct _GelEvalFor {
 	mpw_ptr x;
 	mpw_ptr by;
 	mpw_ptr to;
-	int init_cmp;
+	gint8 init_cmp;
 	GelETree * result;
 	GelETree * body;
 	GelETree * orig_body;
@@ -328,7 +378,8 @@ struct _GelCtx {
 	GelEvalStack *stack;
 	gpointer *topstack;
 	GelETree *res;
-	int post;
+	gboolean post;
+	gboolean whackarg;
 	GelETree *current;
 	mpw_ptr modulo;
 };
